const request = require('request-promise-native');
const cheerio = require('cheerio');
const fake_ua = require('fake-useragent');
const urlencode = require('urlencode');

class Aol {
    optionsDefault = {
        pageOfResult: 1,
        num: 100,
        retry: 3,
        delay: 0,
        resolveWithFullResponse: true,
        jar: true,
        headers: {
            'User-Agent': fake_ua()
        },
    }

    async request(params)
    {
        try {
            var options = this.check(params)
            var options = Object.assign({}, this.optionsDefault, options)
            var response = await this.doRequest(options)
        } catch (error) {
            return new Error("Something errors, please try again later")
        }
        return response;
    }

    async doRequest(options)
    {
        var response = await this.execRequest(options)
        return response
    }

    

    async execRequest(options)
    {
        options = this.pageOfResult(options)
        const response = await request(options)
        return await this.getLinks(response.body)
    }

    async getLinks(body)
    {
        var results = await this.extractLinks(body)
        return results.links

    }

    extractLinks(body)
    {
        var links = [];
        var $ = cheerio.load(body)

        $('body').find('ol.searchCenterMiddle .dd.algo-sr').map((i, data) => {
            var regex = new RegExp(/RU=(.*)\/RK/)

            var title = $(data).find('h3').map((i, h3) => $(h3).text()).get(0)
            var url = $(data).find('h3 a').map((i, url) => $(url).attr('href')).get(0)
            url = urlencode.decode(url.match(regex)[1])
            
            var description = $(data).find('.compText').map((i, desc) => $(desc).text()).get(0)


            links.push({ title, url, description })
        })
        
        return { links }
    }
    pageOfResult(options)
    {
        options.qs.page = parseInt(options.pageOfResult)
        return options
    }
    check(options)
    {
        if ( !options.qs )
            throw new Error("the qs attribute is required")
        if ( !options.qs.q )
            throw new Error("the q attribute is required in qs")

        options.url = this.getUrl(options)
        options.qs.wd = urlencode(options.qs.q)
        
        return options
    }
    getUrl(options)
    {
        return `https://${ options.host || 'search.aol.com' }/aol/search`
    }
}


module.exports = new Aol()