const request = require('request-promise-native');
const cheerio = require('cheerio');
const fake_ua = require('fake-useragent');
const urlencode = require('urlencode');

class Ask {
    optionsDefault = {
        pageOfResult: 1,
        num: 100,
        retry: 3,
        delay: 0,
        resolveWithFullResponse: true,
        jar: true,
        headers: {
            'User-Agent': fake_ua()
        },
    }

    async request(params)
    {
        try {
            var options = this.check(params)
            var options = Object.assign({}, this.optionsDefault, options)
            var response = await this.doRequest(options)
        } catch (error) {
            return new Error("Something errors, please try again later")
        }
        return response;
    }

    async doRequest(options)
    {
        var response = await this.execRequest(options)
        return response
    }

    

    async execRequest(options)
    {
        options = this.pageOfResult(options)
        const response = await request(options)
        return await this.getLinks(response.body)
    }

    async getLinks(body)
    {
        var results = await this.extractLinks(body)
        return results.links

    }

    extractLinks(body)
    {
        var links = [];
        var $ = cheerio.load(body)


        $('body').find('.PartialSearchResults-item-title').map((i, data) => {
            const title = $('body').find('.PartialSearchResults-item-title').map((i, title) => $(title).text() ).get(i).trim()
            const url = $('body').find('.PartialSearchResults-item-title').map((i, title) => $(title).find('a').first().text() ).get(i).trim()
            const description = $('body').find('.PartialSearchResults-item-abstract').map((i, desc) => $(desc).text() ).get(i).trim()

            links.push({ url, title, description });
        })
        return { links }
    }
    pageOfResult(options)
    {
        options.qs.page = parseInt(options.pageOfResult)
        return options
    }
    check(options)
    {
        if ( !options.qs )
            throw new Error("the qs attribute is required")
        if ( !options.qs.q )
            throw new Error("the q attribute is required in qs")

        options.url = this.getUrl(options)
        options.qs.q = urlencode(options.qs.q)
        
        return options
    }
    getUrl(options)
    {
        return `https://${ options.host || 'ask.com' }/web`
    }
}

module.exports = new Ask()