const request = require('request-promise-native');
const cheerio = require('cheerio');
const fake_ua = require('fake-useragent');
const urlencode = require('urlencode');




class Google
{
    optionsDefault = {
        pageOfResult: 1,
        num: 100,
        retry: 3,
        delay: 0,
        resolveWithFullResponse: true,
        jar: true,
        headers: {
            'User-Agent': fake_ua()
        },
    }

    async request(params)
    {
        try {
            
            var options = this.check(params)
            var options = Object.assign({}, this.optionsDefault, options)
            var response = await this.doRequest(options)
        } catch (error) {
            return new Error("Google seems to suspect you as a robot, please try again later");
        }
        return response;
    }

    async doRequest(options)
    {
        var response = await this.execRequest(options)
        return response
    }

    check(options)
    {
        if ( !options.qs )
            throw new Error("the qs attribute is required")
        if ( !options.qs.q )
            throw new Error("the q attribute is required in qs")

        options.url = this.getUrl(options)
        options.qs.q = urlencode(options.qs.q)

        return options
            
    }

    async execRequest(options)
    {
        options = this.pageOfResult(options)
        const response = await request(options)
        return await this.getLinks(response.body)
    }

    async getLinks(body)
    {
        var results = await this.extractLinks(body)
        return results.links

    }

    extractLinks(body)
    {
        var links = [];
        var $ = cheerio.load(body)
        

        $('body').find('.srg h3').each((i, h3) => {
            const href = $(h3).parent().attr('href');
            const description = $('body').find('.srg .s').map((i, s) => {
                return $(s).text()
            }).get(i)
        
            links.push({ url: href, title: $(h3).text(), description: description });
            });
        
        return { links }
    }
    pageOfResult(options)
    {
        options.qs.start = parseInt((options.pageOfResult - 1) * 10) || 0
        return options
    }
    getUrl(options)
    {
        return `https://${ options.host || 'google.com' }/search`
    }
}

module.exports = new Google()