const request = require('request-promise-native');
const cheerio = require('cheerio');
const fake_ua = require('fake-useragent');
const urlencode = require('urlencode');

class Yahoo {
    optionsDefault = {
        pageOfResult: 1,
        num: 100,
        retry: 3,
        delay: 0,
        resolveWithFullResponse: true,
        jar: true,
        headers: {
            'User-Agent': fake_ua()
        },
    }

    async request(params)
    {
        try {
            var options = this.check(params)
            var options = Object.assign({}, this.optionsDefault, options)
            var response = await this.doRequest(options)
        } catch (error) {
            return new Error("Something errors, please try again later");
        }
        return response;
    }

    async doRequest(options)
    {
        var response = await this.execRequest(options)
        return response
    }

    

    async execRequest(options)
    {
        options = this.pageOfResult(options)
        const response = await request(options)
        return await this.getLinks(response.body)
    }

    async getLinks(body)
    {
        var results = await this.extractLinks(body)
        return results.links

    }

    extractLinks(body)
    {
        var links = [];
        var $ = cheerio.load(body)


        $('body').find('ol.searchCenterMiddle li h3').each((i, h3) => {

            var href = $(h3).children().attr('href');
            var regex = new RegExp(/RU=(.*)\/RK/)
            href = urlencode.decode(href.match(regex)[1])

            const description = $('body').find('ol.searchCenterMiddle .compText').map((i, desc) => {
                return $(desc).text()
            }).get(i)
            

            links.push({ url: href, title: $(h3).text(), description: description });
        });
        
        return { links }
    }
    pageOfResult(options)
    {
        options.qs.b = parseInt((options.pageOfResult - 1) * 10) || 0
        return options
    }
    check(options)
    {
        if ( !options.qs )
            throw new Error("the qs attribute is required")
        if ( !options.qs.q )
            throw new Error("the q attribute is required in qs")

        options.qs.p = urlencode(options.qs.q);
        options.url = this.getUrl(options)
        
        return options
    }
    getUrl(options)
    {
        return `https://${ options.host || 'search.yahoo.com' }/search`
    }
}


module.exports = new Yahoo()